package de.intevation.flys.exports.fixings;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WQDay;
import de.intevation.flys.artifacts.model.fixings.FixFunction;
import de.intevation.flys.artifacts.model.fixings.QW;
import de.intevation.flys.exports.ChartGenerator;
import de.intevation.flys.exports.XYChartGenerator;
import de.intevation.flys.jfree.StyledXYSeries;

import java.util.Date;

import org.apache.log4j.Logger;

import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

public class FixWQCurveGenerator 
extends XYChartGenerator 
implements FacetTypes
{
    private static Logger logger =
        Logger.getLogger(FixWQChartGenerator.class);

    public static final String I18N_CHART_TITLE =
        "chart.fixings.wq.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.fixings.wq.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.fixings.wq.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.fixings.wq.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT  =
        "Fixierungsanalyse";

    public static final String I18N_XAXIS_LABEL_DEFAULT  =
        "Q [m\u00B3/s]";

    public static final String I18N_YAXIS_LABEL_DEFAULT  =
        "W [NN + m]";

    public static enum YAXIS {
        W(0),
        Q(1);
        public int idx;
        private YAXIS(int c) {
            idx = c;
        }
    }

    @Override
    public void doOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
        String name = aaf.getFacetName();
        logger.debug("doOut: " + name);

        if(FIX_SECTOR_AVERAGE_WQ.equals(name)) {
            doSectorAverageOut(aaf, doc, visible);
        } else if(FIX_ANALYSIS_EVENTS_WQ.equals(name)) {
            doAnalysisEventsOut(aaf, doc, visible);
        } else if(FIX_ANALYSIS_PERIODS_WQ.equals(name)) {
            doAnalysisPeriodsOut(aaf, doc, visible);
        } else if(FIX_REFERENCE_EVENTS_WQ.equals(name)) {
            doReferenceEventsOut(aaf, doc, visible);
        } else if(FIX_WQ_CURVE.equals(name)) {
            doWQCurveOut(aaf, doc, visible);
        } else if(FIX_OUTLIER.equals(name)) {
            doOutlierOut(aaf, doc, visible);
        } else {
            logger.warn("Unknown facet name " + name);
            return;
        }
    }

    protected void doSectorAverageOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
       logger.debug("doSectorAverageOut");

       XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
       java.util.Random rand = new java.util.Random(7000);
       for(int n = 0; n < 10; n++) {
           series.add(rand.nextDouble(), rand.nextDouble());
       }

       addAxisSeries(series, 0, visible);
    }

    protected void doAnalysisEventsOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
       logger.debug("doAnalysisEventsOut");

       XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
       java.util.Random rand = new java.util.Random(4000);
       for(int n = 0; n < 10; n++) {
           series.add(rand.nextDouble(), rand.nextDouble());
       }

       addAxisSeries(series, 0, visible);
    }

    protected void doAnalysisPeriodsOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
       logger.debug("doAnalysisPeriodsOut");

       XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
       java.util.Random rand = new java.util.Random(5000);
       for(int n = 0; n < 10; n++) {
           series.add(rand.nextDouble(), rand.nextDouble());
       }

       addAxisSeries(series, 0, visible);
    }

    protected void doReferenceEventsOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
       logger.debug("doReferenceEventsOut");

       XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
       java.util.Random rand = new java.util.Random(3000);
       for(int n = 0; n < 10; n++) {
           series.add(rand.nextDouble(), rand.nextDouble());
       }

       addAxisSeries(series, 0, visible);
    }

    protected void doWQCurveOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
        logger.debug("doWQCurveOut");

        FixFunction func = (FixFunction)aaf.getData(context);

        XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
        for (int q = 0; q <= func.getMaxQ(); q += 1.0) {
            series.add(q, func.getFunction().value(q));
        }

        addAxisSeries(series, 0, visible);
    }

    protected void doOutlierOut(ArtifactAndFacet aaf, Document doc, boolean visible) {
        logger.debug("doOutlierOut");

        XYSeries series = new StyledXYSeries(aaf.getFacetDescription(), doc);
        java.util.Random rand = new java.util.Random(2000);
        for(int n = 0; n < 10; n++) {
            series.add(rand.nextDouble(), rand.nextDouble());
        }

        addAxisSeries(series,0, visible);
    }

    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }

    @Override
    protected ChartGenerator.YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {
            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
             public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }   
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
